// tracechar.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "global.h"
#include "TrcLimit.h"
#include "Interval.h"
#include "obj_size_dist.h"
#include "srv_and_size.h"
#include "type_byte.h"

int field=TYPE;

VecPChar vecName;
VecInt vecScale;
int id=0;

char *TypeName[]={
 "HTML",
 "IMAGE",
 "AUDIO",
 "VIDEO",
 "DYNAMIC",
 "FORMATTED",
 "COMPRESSED",
 "PROGRAMS",
 "DIRECTORY",
 "OTHER_TYPES"
};

void Error(char *fmt, ...)
{
  va_list args;

  cerr << "Error: ";

  va_start(args, fmt);
  vfprintf(stderr, fmt, args);
  fprintf(stderr, "\n");
  va_end(args);
    
  exit(1);
}

void readone(FILE *fin, TraceRec &rec)
{
  static int line=0;

  line++;
  if (field==OBJIDSIZE)
  {
    if (2!=fscanf(fin,"%d %d\n",&(rec.objid), &rec.size))
      Error("Wrong input file");
  }
  else
  {
    if (1!=fscanf(fin,"%d\n",&(rec.field[field])))
      Error("Wrong input file");
    rec.objid = id++;
  }
  /*
  if (6!=fscanf(fin,"%d%d%d%d%d%d\n",
    &(rec.interval),&(rec.servicetime),&(rec.objid),
    &(rec.clientid),&(rec.size),&(rec.type)))
  {
    printf("readone() error in %d",line);
    exit(1);
  }*/
}

BOOL greater(int &a, int &b)
{
  return a>b;
}

//For Visual C++
#ifdef  _MSC_VER
#include <crtdbg.h>  //detect memory check
//Check memory leak when in Visual C++
void EnableDebugNew()
{
  // Get current flag
  int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );

  // Turn on leak-checking bit
  tmpFlag |= _CRTDBG_LEAK_CHECK_DF;

  // Turn off CRT block checking bit
  tmpFlag &= ~_CRTDBG_CHECK_CRT_DF;

  // Set flag to the new value
  _CrtSetDbgFlag( tmpFlag );
}
#endif

typedef FILE* PFILE;

void Init(int argc, char* argv[], PFILE &fin, PFILE &fout, BOOL &compress)
{
#ifdef _MSC_VER
//  EnableDebugNew();
#endif

  vecName.push_back("interval");
  vecName.push_back("servicetime");
  vecName.push_back("objid");
  vecName.push_back("clientid");
  vecName.push_back("size");
  vecName.push_back("type");

  vecScale.push_back(1000); // interval
  vecScale.push_back(1000); // servicetime
  vecScale.push_back(1); // objid
  vecScale.push_back(1); // clientid
  vecScale.push_back(1); // size
  vecScale.push_back(1); // type

  if (argc<2 || argc>3)
  {
    printf("Usage: tracechar <tracefile> [outputfile]");
    exit(1);
  }

  if (strstr(argv[1],".gz"))
  {
    char cmd[100];
    compress=TRUE;
    strcpy(cmd,"gzip -cd ");
    strcat(cmd,argv[1]);
    fin=_popen(cmd,"rt");
  }
  else
  {
    compress=FALSE;
    fin=fopen(argv[1],"rt");
  }
  if (fin==NULL)
  {
    printf("Error open file.");
    exit(1);
  }

  if (argc==3)
  {
    if ((fout=fopen(argv[2],"at"))==NULL)
    {
      printf("Error create file.");
      exit(1);
    }
  }
  else
    fout = stdout;
}

int main(int argc, char* argv[])
{
  VectorPScanTrace vecPScanTrace;
  FILE *fin, *fout;
  BOOL compress;
  int i;
  TraceRec rec;
  Init(argc, argv, fin, fout, compress);

  for (i=0; i<MAX; i++)
    rec.field[i]=0;

  vecPScanTrace.push_back(new TypeByte(fout));
//  vecPScanTrace.push_back(new TrcLimit(fout));
//  vecPScanTrace.push_back(new SrvAndSize(fout));
//  vecPScanTrace.push_back(new ObjSizeDist(fout));

  //vecPScanTrace.push_back(new Interval(fout));

  struct tm *newtime;
  time_t aclock;
  time( &aclock );                 /* Get time in seconds */
  newtime = localtime( &aclock );  /* Convert time to struct */
                                   /* tm form */
  /* Print local time as a string */
  fprintf(fout, "%s", asctime( newtime ) );

  time_t preTime, interval, beginTime;
  int elapsedTime;
  beginTime = preTime = time(NULL);
  interval = 30; // seconds

  int line=0;
  while (!feof(fin))
  {
    if (time(NULL)-preTime >= interval)
    {
      preTime = time(NULL);
      elapsedTime = preTime-beginTime;
      printf("%d:%02d line:%d\n",elapsedTime/60, elapsedTime%60, line);
    }

    line++;
    readone(fin, rec);
    for (i=0; i<vecPScanTrace.size(); i++)
      vecPScanTrace[i]->process_one(rec);

#ifdef _DEBUG
    if (line>1000) // for debug
      break;
#endif
  }

  for (i=0; i<vecPScanTrace.size(); i++)
    delete vecPScanTrace[i];

  if (compress)
    _pclose(fin);
  else
    fclose(fin);
  if (argc==3)
    fclose(fout);

  int t = clock()/CLOCKS_PER_SEC;
  printf("Run time: %d min %d sec\n",t/60,t%60);
  printf("\a");
  return 0;
}
