// SingleDistrib.cpp: implementation of the SingleDistrib class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SingleDistrib.h"
#include "DistribFunc.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SingleDistrib::SingleDistrib()
{
}

SingleDistrib::~SingleDistrib()
{

}

double SingleDistrib::next()
{
  double rand;
  double r;

  rand = double(m_rand.random())/INT_MAX;
  if (m_pfunc==NULL) // random 
  {
    r = rand*(m_to - m_from) + m_from;
  }
  else if (m_pfunc == DistribFunc::Zipf) // search, get result
  {
    int f,l,mid;
    f=1;
    l=m_cdf.size()-1;
    while (TRUE)
    {
      mid=(l+f)/2;
      if (rand > m_cdf[mid])
        f=mid;
      else
        l=mid;
      if (l==f || f+1==l)
        break;
    }
    r=l;
  }
  else if (m_pfunc == DistribFunc::Pareto)
  {
    r = m_pfunc(m_param1, rand, m_param2)*m_from;
    while (r>m_to)
    {
      rand = double(m_rand.random())/INT_MAX;
      r = m_pfunc(m_param1, rand, m_param2)*m_from;
      if (r<m_from)
        r=r;
    }
  }
  else if (m_pfunc == DistribFunc::Equ) 
  {
    r = m_pfunc(m_param1, rand, m_param2);
  }
  return r;
}

void SingleDistrib::ReadConfig(char line[])
{
  istrstream *pistr;
  char sDistribName[10];

  m_param1=m_param2=0;
  pistr = new istrstream(line);
  *pistr >> m_from >> m_to >> m_prob >> sDistribName >> m_param1 >> m_param2;
  delete pistr;

  m_prob /= 100.0;

  m_rand.srandom(GetSeed());

  if (stricmp(sDistribName,"Exp")==0)
  {
    m_pfunc = DistribFunc::Exponential;
  }
  else if (stricmp(sDistribName,"Par")==0)
  {
    m_pfunc = DistribFunc::Pareto;
  }
  else if (stricmp(sDistribName,"Equ")==0)
  {
    m_pfunc = DistribFunc::Equ;
  }
  else if (stricmp(sDistribName,"Ran")==0)
  {
    m_pfunc = NULL;
  }
  else if (stricmp(sDistribName,"Zip")==0)
  {
    m_pfunc = DistribFunc::Zipf;
  }
  else if (stricmp(sDistribName,"Clientid")==0)
  {
    m_pfunc = DistribFunc::Clientid;
  }

  if (m_pfunc==DistribFunc::Zipf)
  {
    // generate function table
    int i, num;
    double sum=0, parsum=0, d;
    num = 500000;
    m_rank.push_back(0);
    m_cdf.push_back(0);
    for (i=1; i<num; i++)
    {
      d=double(m_param2)/pow(i,m_param1);
      m_rank.push_back(d);
      sum += m_rank[i];
    }
    for (i=1; i<num; i++)
    {
      parsum += m_rank[i];
      m_cdf.push_back(parsum/sum);
    }
    m_rank.erase(m_rank.begin(), m_rank.end());
  }
}
