// SynTrace.cpp: implementation of the SynTrace class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SynTrace.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SynTrace::SynTrace(int seed)
{
  m_seed = seed;
}

SynTrace::~SynTrace()
{
  int i;
  
  for (i=0; i<m_vecPWholeDistrib.size(); i++)
    delete m_vecPWholeDistrib[i];
}

void SynTrace::ReadConfig(char sFileName[])
{
  ifstream cfgFile;
  char line[255], name[255], s[255];
  int i;
  PWholeDistrib pWholeDistrib, pObjid, pSize;

  cfgFile.open(sFileName, ios::in);
  if (!cfgFile)
    Error("Config file %s not exist!", sFileName);

  ReadLine(cfgFile, line, 254);
  i=sscanf(line,"Num=%d",&m_numTrace);
  if (i!=1)
    Error("Num= <number> expected!");

  InitSeed(m_seed, m_numTrace);

  ReadLine(cfgFile, line, 254);
  i=sscanf(line,"File=%s",m_sTraceName);
  if (i!=1)
    Error("File= <file name> expected!");

  pObjid=pSize=NULL;
  while (TRUE)
  {
    ReadLine(cfgFile,line,254);
    // general section
    if (strstr(line,"[End]")) // config finishes
    {
      break;
    }

    i = sscanf(line, "%s",s);
    if (i!=1 || s[0]!='[' || s[strlen(s)-1]!=']')
      Error("[name] expected in config file!");

    s[strlen(s)-1]=0;
    strcpy(name,s+1);

    pWholeDistrib = new WholeDistrib(name);
    pWholeDistrib->ReadConfig(cfgFile);
    if (stricmp(name,"Objid")==0)
      pObjid=pWholeDistrib;
    else if (stricmp(name,"Size")==0)
      pSize=pWholeDistrib;
    m_vecPWholeDistrib.push_back(pWholeDistrib);
  }
  if (pObjid && pSize)
    pSize->m_pObjid = pObjid;
}

void SynTrace::GenTrace()
{
  ofstream fTrace;
  int i, j;
  double d;
  PWholeDistrib pWholeDistrib;

  fTrace.open(m_sTraceName);
  if (!fTrace)
    Error("Create %s fail!", m_sTraceName);

  for (i=0; i<m_numTrace; i++)
  {
    for (j=0; j<m_vecPWholeDistrib.size(); j++)
    {
      pWholeDistrib = m_vecPWholeDistrib[j];
      d = pWholeDistrib->next();
      fTrace << int(d) << '\t';
    }
    fTrace << endl;
  }
}
