// WholeDistrib.cpp: implementation of the WholeDistrib class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "WholeDistrib.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

WholeDistrib::WholeDistrib(char sName[])
{
  strcpy(m_sName, sName);

  m_pObjid = NULL;
  m_prev = 0;
  if (stricmp(m_sName,"INTERVAL")==0)
    m_id=INTERVAL;
  else if (stricmp(m_sName,"SERVICETIME")==0)
    m_id=SERVICETIME;
  if (stricmp(m_sName,"ObjId")==0)
    m_id=OBJID;
  else if (stricmp(m_sName,"CLIENTID")==0)
    m_id=CLIENTID;
  else if (stricmp(m_sName,"Size")==0)
    m_id=SIZE;
  else if (stricmp(m_sName,"TYPE")==0)
    m_id=TYPE;
}

WholeDistrib::~WholeDistrib()
{
  int i;
  for (i=0; i<m_vecPSingleDistrib.size(); i++)
    delete m_vecPSingleDistrib[i];
}

void WholeDistrib::ReadConfig(ifstream &fCfg)
{
  char line[255];
  char *p;
  PSingleDistrib pSingleDistrib;

  while (TRUE)
  {
    ReadLine(fCfg,line,254);
    if (strstr(line,"End"))
      break;
    p=strchr(line,'=');
    if (p==NULL)
      Error("Error format in %s",m_sName);

    pSingleDistrib = new SingleDistrib();
    pSingleDistrib->ReadConfig(p+1);
    m_vecPSingleDistrib.push_back(pSingleDistrib);
  }
  m_rand.srandom(GetSeed());
}

int WholeDistrib::next()
{
  double d, p;
  int r;
  int i;

  if (m_pObjid) // find the old size if exists
  {
    MapInt2Int::iterator it;
    it = m_MapObjid.find(m_pObjid->m_prev);
    if (it!=m_MapObjid.end())
      return it->second;
  }
  d=double(m_rand.random())/INT_MAX;
  p=0;

  for (i=0; i<m_vecPSingleDistrib.size(); i++)
  {
    p += m_vecPSingleDistrib[i]->prob();
    if (p>d)
    {
      r = int(m_vecPSingleDistrib[i]->next());
      m_prev = r;
      if (m_pObjid)
      {
        m_MapObjid.insert(MapInt2Int::value_type(m_pObjid->m_prev,r));
      }
      return r;
    }
  }
  Error("Error in WholeDistrib::next()!");
  return 0;
}
