// TrcLimit.cpp: implementation of the TrcLimit class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TrcLimit.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TrcLimit::TrcLimit(FILE *fout)
  :ScanTrace(fout)
{
  int i;
  for (i=0; i<MAX; i++)
  {
    vecMax.push_back(0);
    vecPVecCumu.push_back(new VecInt);
  }
  sum=0;
  intervalmean=15471.260250;
  intervalstdv=0;
}

TrcLimit::~TrcLimit()
{
  FILE *f;
  char name[50];
  int i,j,m;
  int oneref=0;
  BOOL needsort,needcumu;

  printf("Number of traces:%d\n",sum);

  for (i=0; i<MAX; i++)
  {
    fprintf(m_fout,"Max %s=%d\n",vecName[i],vecMax[i]);
  }
  
  for (i=0; i<MAX; i++)
  {
    fprintf(m_fout,"distinct num of %s=%d\n",vecName[i],nonzero(*(vecPVecCumu[i])));
  }

  // write distribution
  for (i=0; i<MAX; i++)
  {
    needsort=FALSE;
    needcumu=FALSE;
    PVecInt pv = vecPVecCumu[i];
    VecDouble v;
    switch (i)
    {
    case OBJID:
      needsort=TRUE;
      needcumu=TRUE;
      for (j=0; j<pv->size(); j++)
      {
        if ((*pv)[j]==1)
          oneref++;
      }
      fprintf(m_fout,"One time referencing: %d\n",oneref);
      stats(*pv);
      break;
    case CLIENTID:
      needsort=TRUE;
      needcumu=TRUE;
      break;
    case SIZE:
      //needsort=TRUE;
      needcumu=TRUE;
      break;
    case SERVICETIME:
      break;
    case INTERVAL:
      intervalstdv = sqrt(intervalstdv/(sum-1));
      fprintf(m_fout,"Stdv of interval: %lf\n",intervalstdv);
      break;
    case TYPE:
      break;
    default:
      assert(FALSE);
    }
  

    if (needsort)
    {
      sort(pv->begin(), pv->end(), greater);
    }
    if (needcumu)
    {
      // compute accumulative num
      v.push_back((*pv)[0]);
      for (j = 1; j<pv->size(); j++) 
      {
        m=v[j-1] + (*pv)[j];
        v.push_back(m);
      }
      for (j = 0; j<v.size(); j++)
        v[j] /= sum;  

      strcpy(name,vecName[i]);
      strcat(name,"Cdf.txt");
      if ((f=fopen(name,"wt"))==NULL)
      {
        printf("Create %s fail.\n",name);
        exit(1);
      }
      for (j=0; j<v.size(); j++)
      {
        if (v[j]==0)
          continue;
        fprintf(f,"%d %lf\n",j,v[j]);
      }
      fclose(f);
    }
    else
    {
      strcpy(name,vecName[i]);
      strcat(name,"Dist.txt");
      if ((f=fopen(name,"wt"))==NULL)
      {
        printf("Create %s fail.\n",name);
        exit(1);
      }
      for (j=0; j<pv->size(); j++)
      {
        if (pv->at(j)==0)
          continue;
        fprintf(f,"%d %d\n",j,pv->at(j));
      }
      fclose(f);
    }
  }

  for (i=0; i<MAX; i++)
  {
    delete vecPVecCumu[i];
  }
}

TrcLimit::process_one(TraceRec &rec)
{
  int i,m;
  sum++;
  for (i=0; i<MAX; i++)
  {
    if (i==INTERVAL)
    {
      m=intervalmean-rec.field[i];
      intervalstdv += m*m;
    }
    tomax(vecMax[i], rec.field[i]);
    assert(sum>0);
    vecinc(*(vecPVecCumu[i]), rec.field[i]/vecScale[i]);
  }
}
