// TrcLimit.h: interface for the TrcLimit class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TRCLIMIT_H__580B6DF5_D8BE_11D3_9E5D_002035681F74__INCLUDED_)
#define AFX_TRCLIMIT_H__580B6DF5_D8BE_11D3_9E5D_002035681F74__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ScanTrace.h"

struct File{
  int objid;
  int size;
};
typedef vector<File> VecFile;

BOOL fileGreater(File &a, File &b)
{
  return a.objid>b.objid;
}

BOOL smallerSize(File &a, File &b)
{
  return a.size<b.size;
}

class FileIdSize : public ScanTrace
{
  VecFile vecFile;

public:
  FileIdSize(FILE *fout) : ScanTrace(fout)
  {
  };

  virtual ~FileIdSize()
  {
    ofstream fout;
    int i;
    sort(vecFile.begin(), vecFile.end(), fileGreater);

    double xysum=0, refsum=0, sizesum=0, num=0;
    double refsqrsum=0, sizesqrsum=0;
    double xymean, refmean, sizemean, cor;
    double refvar, sizevar;
    int ref, size;

    for (i=0; i<vecFile.size(); i++)
    {
      if (vecFile[i].objid==0)
        continue;
      ref=vecFile[i].objid;
      size=vecFile[i].size;
      num ++;
      refsum += 1.0*ref;
      refsqrsum += 1.0*ref*ref;
      sizesum += 1.0*size;
      sizesqrsum += 1.0*size*size;
      xysum += 1.0*ref*size;
    }
    
    xymean = xysum/num;
    refmean=refsum/num;
    sizemean=sizesum/num;
    refvar = (refsqrsum-num*refmean*refmean)/num;
    sizevar = (sizesqrsum-num*sizemean*sizemean)/num;
    cor = (xymean-refmean*sizemean)/sqrt(refvar*sizevar);

    fprintf(m_fout,"Mean refCount:%lg  size:%lg xy:%lg cor:%lg\n",
      refmean, sizemean, xymean, cor);

    fout.open("obj_size.txt");
    if (!fout)
    {
      printf("Create file error!");
      exit(1);
    }
    for (i=0; i<vecFile.size(); i++)
    {
      if (vecFile[i].size==0)
        continue;
      fout << i << '\t' << vecFile[i].size << endl;
    }
    fout.close();

    double sum=0, cdf=0;
    int rank=0;
    sort(vecFile.begin(), vecFile.end(), smallerSize);
    fout.open("size_freq_dist.txt");
    if (!fout)
    {
      printf("Create file error!");
      exit(1);
    }
    for (i=0; i<vecFile.size(); i++)
    {
      if (vecFile[i].size==0)
        continue;
      rank++;
      sum+=vecFile[i].objid;
      fout << rank << '\t' << vecFile[i].objid << endl;
    }
    fout.close();

    rank=0;
    fout.open("size_freq_cdf.txt");
    if (!fout)
    {
      printf("Create file error!");
      exit(1);
    }
    for (i=0; i<vecFile.size(); i++)
    {
      if (vecFile[i].size==0)
        continue;
      rank++;
      cdf += vecFile[i].objid;
      fout << rank << '\t' << cdf/sum << endl;
    }
    fout.close();

    vecFile.erase(vecFile.begin(), vecFile.end());
  };

  virtual void pre_process(TraceRec &rec)
  {
  }

  virtual void process_one(TraceRec &rec)
  {
    int idx;
    int i, s;
    File fileRec;
    fileRec.size=fileRec.objid=0;

    idx=(rec.objid+vecScale[OBJID]/2)/vecScale[OBJID];

    s=vecFile.size();
    if (idx>s-1)
    {
      for (i=0; i<=idx-s; i++)
        vecFile.push_back(fileRec); // default value
    }

    vecFile[idx].objid++;
    vecFile[idx].size=rec.size;
  }
};

class TrcLimit : public ScanTrace  
{
protected:
  VecInt vecMax;
  int sum;
  VecPVecInt vecPVecCumu;

  VecDouble vecSum, vecSqrSum, vecMean, vecCov;
  VecInt64 vecNum;
  VecInt srv_size, file_size;
  double delta, min, max;

  void stats(char name[], VecInt &v)
  {
    int m, i, j, num;
    double dVar, dMean, dStdv;
    num=0;
    for (i=0; i<v.size(); i++)
      if (v[i]>0)
        num++;
    dMean = double(sum)/num;
    dVar=0;
    for (j=0; j<v.size(); j++)
    {
      dVar += (double(v[i])-dMean)*(double(v[i])-dMean);
    }
    dVar /= num;
    dStdv = sqrt(dVar);
    m=0;
    for (j=0; j<v.size(); j++)
    {
      m += v[j];
      if (m*2>num)
        break;
    }
    fprintf(m_fout,"%s Median:%d  mean:%lf  stdv:%lf  cov:%lf\n", 
            name, m, dMean, dStdv, dStdv/dMean);
  };
public:
	TrcLimit(FILE *fout) : ScanTrace(fout)
  {
    int i;
    for (i=0; i<MAX; i++)
    {
      vecMax.push_back(0);
      vecPVecCumu.push_back(new VecInt);
      vecMean.push_back(0);
      vecCov.push_back(0);
      vecSqrSum.push_back(0);
      vecSum.push_back(0);
      vecNum.push_back(0);
    }
    sum=0;
    delta = 100;
    min = 1e100;
    max = 1e-100;
  };

  virtual ~TrcLimit()
  {
    printf("Deleting TrcLimit...\n");

    FILE *f;
    char name[50];
    int i,j;
    INT64 m;
    int oneref=0;
    BOOL needsort,needcumu;
    double prev;

    printf("Number of traces:%d\n",sum);

    for (i=0; i<MAX; i++)
    {
      fprintf(m_fout,"Max %s=%d\n",vecName[i],vecMax[i]);
      fprintf(m_fout,"Sum %s=%lg\n",vecName[i],vecSum[i]);
    }
  
    for (i=0; i<MAX; i++)
    {
      fprintf(m_fout,"distinct num of %s=%d\n",vecName[i],nonzero(*(vecPVecCumu[i])));
    }

    // write distribution
    for (i=0; i<MAX; i++)
    {
      if (vecNum[i]!=0)
      {
        vecMean[i] = vecSum[i]/vecNum[i];
        if (vecMean[i]!=0)
        {
          vecCov[i] = sqrt((vecSqrSum[i]-vecNum[i]*vecMean[i]*vecMean[i])/
                    vecNum[i])/vecMean[i];
        }
      }
      needsort=FALSE;
      needcumu=FALSE;
      PVecInt pv = vecPVecCumu[i];
      VecDouble v;
      VecInt64 vi64;

      INT64 prevy=0;
      INT64 thesum=0;
      int n;
      n=1;
      for (j=0; j<pv->size(); j++)
      {
        if ((*pv)[j]==0)
          continue;
        if ((*pv)[j] == prevy)
        {
          thesum += j;
          n++;
          (*pv)[j]=0;
        } 
        else
        {
          (*pv)[thesum/n] = prevy;
          thesum = j; 
          n =1; 
          prevy = (*pv)[j];
        }
      }

      switch (i)
      {
      case OBJID:
        {
        sort(pv->begin(), pv->end(), greater);
        needcumu=TRUE;
        int sum=0;
        for (j=0; j<pv->size(); j++)
        {
          sum += (*pv)[j];
          if ((*pv)[j]==1)
            oneref++;
        }
        fprintf(m_fout,"One time referencing: %d, %lg\n",
            oneref, double(oneref)/sum);
        stats(vecName[i], *pv); // print median, mean, stdv, cov
        break;
        }
      case CLIENTID:
        sort(pv->begin(), pv->end(), greater);
        needcumu=TRUE;
        stats(vecName[i], *pv); // print median, mean, stdv, cov
        break;
      case SIZE:
        {
          INT64 t,n,m;
          double d;

          m=0;
          for (j=0; j<pv->size(); j++)
          {
            m += (*pv)[j];
            if (m*2>sum)
              break;
          }
          fprintf(m_fout,"Transfered median of size: %d\n",j*vecScale[i]);

          // compute accumulative num
          if ((f=fopen("sizebytedist.txt","wt"))==NULL)
          {
            printf("Create %s fail.\n","sizebytecdf.txt");
            exit(1);
          }

          vi64.push_back(0);
          t=0;
          for (j = 1; j<pv->size(); j++) 
          {
            n = (*pv)[j]*j*vecScale[SIZE];
            if (n!=0)
              fprintf(f,"%d %I64d\n",j,n);
            t += n;
            m = vi64[j-1] + n;
            if (m<0 || m!=t)
            {
              printf("Big size!");
              exit(1);
            }
            vi64.push_back(m);
          }
          fclose(f);

          if ((f=fopen("sizebytecdf.txt","wt"))==NULL)
          {
            printf("Create %s fail.\n","sizebytecdf.txt");
            exit(1);
          }
          INT64 prev=0;
          for (j = 0; j<vi64.size(); j++)
          {
            d = double(vi64[j]) / vecSum[SIZE];
            if (vi64[j]==prev)
              continue;
            fprintf(f,"%d %lg\n",j*vecScale[SIZE],d);
            prev=vi64[j];
          }
          fclose(f);

          vi64.erase(vi64.begin(),vi64.end());

          fprintf(m_fout,"%s: transfered mean: %lf  ", vecName[i], vecMean[i]);
          fprintf(m_fout,"COV:%lf  ", vecCov[i]);
          fprintf(m_fout,"Stdv:%lf\n",vecCov[i]*vecMean[i]);
          //sort(pv->begin(), pv->end(), greater);
          needcumu=TRUE;
          break;
        }
      case SERVICETIME:
        fprintf(m_fout,"%s: Mean: %lf  ", vecName[i], vecMean[i]);
        fprintf(m_fout,"COV:%lf  ", vecCov[i]);
        fprintf(m_fout,"Stdv:%lf\n",vecCov[i]*vecMean[i]);
        break;
      case INTERVAL:
        {
          needcumu=TRUE;
          m=0;
          for (j=0; j<pv->size(); j++)
          {
            m += (*pv)[j];
            if (m*2>sum)
              break;
          }
          fprintf(m_fout,"Median of interval: %d\n",j*vecScale[i]);
          m=0;
          for (j=pv->size()-1; j>=0; j--)
          {
            m += (*pv)[j];
            if (m*2>sum)
              break;
          }
          fprintf(m_fout,"%s: Mean: %lf  ", vecName[i], vecMean[i]);
          fprintf(m_fout,"COV:%lf  ", vecCov[i]);
          fprintf(m_fout,"Stdv:%lf\n",vecCov[i]*vecMean[i]);

          {
            double mean, sum, sqrsum, var, stdv;
            prev=0;
            mean=sum=sqrsum=0;
            for (j=0; j<pv->size(); j++)
            {
              sum += 1.0*(*pv)[j]*vecScale[i]*j;
              sqrsum += 1.0*j*vecScale[i]*j*vecScale[i]*(*pv)[j];
            }
            mean = sum/vecNum[i];
            var = (sqrsum-vecNum[i]*mean*mean)/vecNum[i];
            stdv = sqrt(var);
            fprintf(m_fout,"%s: NewMean: %lf  ", vecName[i], mean);
            fprintf(m_fout,"newCOV:%lf  ", stdv/mean);
            fprintf(m_fout,"newStdv:%lf\n",stdv);
          }

          break;
        }
      case TYPE:
        break;
      default:
        assert(FALSE);
      }

      strcpy(name,vecName[i]);
      strcat(name,"Dist.txt");
      if ((f=fopen(name,"wt"))==NULL)
      {
        printf("Create %s fail.\n",name);
        exit(1);
      }
      prev=0;
      for (j=0; j<pv->size(); j++)
      {
        if (pv->at(j)==0)
          continue;
        if (pv->at(j)-prev==0)
          continue;
        fprintf(f,"%d %d\n",j*vecScale[i],pv->at(j));
        prev=pv->at(j);
      }
      fclose(f);

      strcpy(name,vecName[i]);
      strcat(name,"Pdf.txt");
      if ((f=fopen(name,"wt"))==NULL)
      {
        printf("Create %s fail.\n",name);
        exit(1);
      }
      prev=0;
      for (j=0; j<pv->size(); j++)
      {
        if (pv->at(j)==0)
          continue;
        if (pv->at(j)==prev)
          continue;
        fprintf(f,"%d %lg\n",j*vecScale[i], 1.0*pv->at(j)/vecSum[i]);
        prev=pv->at(j);
      }
      fclose(f);

      if (i==SIZE)
      {
        strcpy(name,"size_freq_dist.txt");
        if ((f=fopen(name,"wt"))==NULL)
        {
          printf("Create %s fail.\n",name);
          exit(1);
        }

        int s,sum, cur;
        s=100;
        sum=0;
        cur=0;
        for (j = 0; j<pv->size(); j++) 
        {
          if (j< cur*s+s/2)
          {
            sum += (*pv)[j];
          }
          else
          {
            if (sum>0)
              fprintf(f,"%d %d\n",cur*s,sum);
            cur++;
            sum=0;
          }
        }

        fclose(f);
      }
      if (needcumu)
      {
        // compute accumulative num
        v.push_back((*pv)[0]);
        sum=0;
        for (j = 1; j<pv->size(); j++) 
        {
          sum+=(*pv)[j];
          m=v[j-1] + (*pv)[j];
          v.push_back(m);
        }
        for (j = 0; j<v.size(); j++)
          v[j] /= sum;  

        if (i==SIZE)
        {
          strcpy(name,vecName[i]);
          strcat(name,"1-Cdf.txt");
          if ((f=fopen(name,"wt"))==NULL)
          {
            printf("Create %s fail.\n",name);
            exit(1);
          }

          double prevy=0;
          double thesum=0;
          double cdf=0;
          int n;
          for (j=0; j<v.size(); j++)
          {
            if (v[j]==0)
              continue;
            if (v[j] == prevy)
            {
              thesum += j;n++;
            } 
            else
            {
              double d=prevy;
              fprintf(f,"%d %lg\n", int(thesum/n*vecScale[i]), d);
              thesum = j; 
              n =1; 
              prevy =v[j];
            }
          }
          fclose(f);
        }

        if (i==OBJID || i==CLIENTID)
        {
          strcpy(name,vecName[i]);
          strcat(name,"NorCdf.txt");
          if ((f=fopen(name,"wt"))==NULL)
          {
            printf("Create %s fail.\n",name);
            exit(1);
          }
          prev=0;
          for (j=0; j<v.size(); j++)
          {
            if (v[j]==prev)
              continue;
            fprintf(f,"%lg %lg\n",1.0*(j+1)*vecScale[i]/v.size(),v[j]);
            prev=v[j];
          }
          fclose(f);
        }
        
        strcpy(name,vecName[i]);
        strcat(name,"ReqCdf.txt");
        if ((f=fopen(name,"wt"))==NULL)
        {
          printf("Create %s fail.\n",name);
          exit(1);
        }
        prev=0;
        for (j=0; j<v.size(); j++)
        {
          if (v[j]==prev)
            continue;
          if (i==INTERVAL)
            fprintf(f,"%d %lg\n",j*vecScale[i],v[j]);
          else
            fprintf(f,"%d %lg\n",(j+1)*vecScale[i],v[j]);
          prev=v[j];
        }
        fclose(f);

        v.erase(v.begin(),v.end());
      }
      pv->erase(pv->begin(),pv->end());
    }

    for (i=0; i<MAX; i++)
    {
      delete vecPVecCumu[i];
    }

    // file_size distribution
    double size, distsize;
    INT64 sum;
    if ((f=fopen("obj_size_cdf.txt","wt"))==NULL)
    {
      printf("Create %s error!\n","obj_size_cdf.txt");
      exit(1);
    }
    size=distsize=0;
    int distinctnum=0;
    double distinctsqrsum=0, distinctsum=0;
    double distinctvar, distinctstdv, distinctmean;
    for (i=0; i<file_size.size(); i++)
    {
      if (file_size[i]==0)
        continue;
      distinctnum++;
      distinctsqrsum += double(file_size[i])*i*i;
      distinctsum += file_size[i]*i;
      size += file_size[i]*i;
      distsize += file_size[i];
    }
    distinctmean = distinctsum/distinctnum;
    distinctvar = (distinctsqrsum-distinctmean*distinctmean*distinctnum)/distinctnum;
    distinctstdv = sqrt(distinctvar);
    printf("Distinct file size: num: %d  mean:%lg  stdv:%lg  cov:%lg\n", 
          distinctnum, distinctmean, distinctstdv, distinctstdv/distinctmean);

    m=0;
    for (j=0; j<file_size.size(); j++)
    {
      m += file_size[j];
      if (m*2>distinctnum)
        break;
    }
    fprintf(m_fout,"Uniq median of size: %d\n",j*vecScale[SIZE]);

    sum=0;
    for (i=0; i<file_size.size(); i++)
    {
      if (file_size[i]==0)
        continue;
      sum += file_size[i];
      fprintf(f,"%d %lg\n",i*vecScale[SIZE],sum/distsize);
    }
    fclose(f);
    if ((f=fopen("size_llcd.txt","wt"))==NULL)
    {
      printf("Create %s error!\n","obj_size_cdf.txt");
      exit(1);
    }
    sum=0;
    for (i=0; i<file_size.size(); i++)
    {
      if (file_size[i]==0)
        continue;
      sum += file_size[i];
      fprintf(f,"%d %lg\n",i*vecScale[SIZE],1.0-sum/distsize);
    }
    fclose(f);
    file_size.erase(file_size.begin(),file_size.end());
  };

  virtual void pre_process(TraceRec &rec)
  {
  }

  virtual void process_one(TraceRec &rec)
  {
    int i;
    sum++;

    for (i=0; i<MAX; i++)
    {
      switch (i)
      {
      case INTERVAL:
      case SERVICETIME:
      case SIZE:
        {
          vecSum[i] += rec.field[i];
          vecNum[i]++;
          vecSqrSum[i] += double(rec.field[i])*rec.field[i];
          if ((vecSqrSum[i]<0) || (vecSum[i]<0))
          {
            printf("Too big! %lf %d",vecSqrSum[i],vecSum[i]);
            exit(1);
          }
          break;
        }
      case CLIENTID:
      case OBJID:
      case TYPE:
        {
          vecNum[i]++;
          vecSum[i]++;
          break;
        }
      default:
        break;
      }
      tomax(vecMax[i], rec.field[i]);
      assert(sum>0);
      vecinc(*(vecPVecCumu[i]), (rec.field[i]+vecScale[i]/2)/vecScale[i]);
    }
    if (vecPVecCumu[OBJID]->at(rec.objid)==1) // this file has not been counted
      vecinc(file_size, rec.size);
  };
};

#endif // !defined(AFX_TRCLIMIT_H__580B6DF5_D8BE_11D3_9E5D_002035681F74__INCLUDED_)
